package de.cnam.jemina.persistents;

import de.cnam.jemina.extern.auth.IUser;
import de.cnam.jemina.objects.Artikel;
import de.cnam.jemina.objects.Filiale;
import de.cnam.jemina.objects.Person;
import de.cnam.jemina.objects.Reservierung;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

/**
 * Schnittstelle zur Persistenzklasse
 * @author CNAM Team (Nadine, Mike, Jens)
 * @since 22.01.2013
 */
public interface IPersistent {
    
    /**
     * Speichert das &uuml;bergebene Objekt ab (erstmalige Speicherung)
     * @param obj Das zu speichernde Objekt
     * @param user Benutzer, der die Speicherung durchf&uuml;hrt
     * @throws SQLException 
     */
    public void insert(IPersistensObject obj, IUser user) throws SQLException;

    /**
     * &Auml;ndert das &uuml;bergebene Objekt
     * @param obj Das zu &auml;nderte Objekt
     * @param user Benutzer, der die &Auml;nderung durchf&uuml;hrt
     * @throws SQLException 
     */
    public void update(IPersistensObject obj, IUser user) throws SQLException;
    
    /**
     * L&ouml;scht das &uuml;bergebene Objekt
     * @param obj Das zu l&ouml;schende Objekt
     * @param user Benutzer, der die L&ouml;schung durchf&uuml;hrt
     * @throws SQLException 
     */
    public void delete(IPersistensObject obj, IUser user) throws SQLException;
    
    /**
     * Liefert alle Artikel die zu dem &uuml;bergebenen Suchbegriff passen.
     * @param searchtxt Suchtext (kann null sein)
     * @return Liefert eine leere Liste, falls keine passenden Artikel gefunden wurden
     * @throws SQLException 
     */
    public List<Artikel> selectArticle(String searchtxt) throws SQLException;
    
    /**
     * Liefert den Artikel mit der &uuml;bergeben ID
     * @param id Artikel-ID
     * @return
     * @throws SQLException 
     */
    public Artikel loadArtikelDetails(int id) throws SQLException;
    
    
    /**
     * Liefert alle Artikel die zu dem &uuml;bergebenen Suchbegriff passen.
     * Zus&auml;tzlich werden Standortinformationen mit &uuml;bergeben
     * @param searchtxt Suchtext (kann null sein)
     * @return Liefert eine leere Liste, falls keine passenden Artikel gefunden wurden
     * @throws SQLException 
     */
//    public List<ArtikelListObject> selectArticleWithLocation(String searchtxt) throws SQLException;
    
    /**
     * L&ouml;st eine Reservierung aus.
     * Sollte dies nicht m&ouml;glich sein, wird <b>null</b> zur&uuml;ckgegeben
     * @param a Zu reservierender Artikel
     * @param f Filiale bei der der Artikel reserviert werden sol
     * @param u User-Objekt
     * @return <b>null</b> wenn keine Reservierung m&ouml;glich ist
     * @throws SQLException 
     * @todo: Langfristig sollte {@link #insert(de.cnam.jemina.persistents.IPersistensObject, de.cnam.jemina.extern.auth.IUser)} mit dem entsprechendem Objekt aufgerufen werden
     */
    public String reserve(Artikel a, Filiale f, IUser u) throws SQLException;
    
    
    /**
     * L&ouml;scht eine Reservierung anhand der ID
     * @param id Reservierungs-ID
     * @throws SQLException 
     */
//    public void delReserve(int id) throws SQLException;
    
    
    /**
     * L&ouml;scht eine Reservierung anhand der Reservierungsnr
     * @param number Reservierungs-Nummer
     * @throws SQLException 
     */
    public void delReserve(String number) throws SQLException;
    
    
    /**
     * Liefert die {@link Reservierung} zu einer eingegeben Nummer.
     * <b>Nicht case sensitiv</b>
     * @param number Reservierungsnummer
     * @return Liefert <b>null</b> wenn keine Reservierung gefunden wurde
     * @throws SQLException 
     */
    public Reservierung getReserve(String number) throws SQLException;
    
    /**
     * Leiht ein Exemplar aus, dabei wird die Reservierung gel&ouml;scht
     * 
     * @param r Reservierung
     * @param exemplarId ID des Exemplares
     * @throws SQLException 
     */
    public void loan(Reservierung r, int exemplarId) throws SQLException;
    
    
    /**
     * Liefert ein passendes {@link Person} Objekt zu dem &uuml;bergeben User.
     * Ist keines vorhanden, wird dieses angelegt
     * @param u User
     * @return
     * @throws SQLException 
     */
    public Person getPerson(IUser u) throws SQLException;
    
    /**
     * Diese Methode f&uuml;hrt den &uuml;bergebenen SQL-Befehl aus.
     * Dies wird u.a. ben&ouml;tigt um Testdaten zu importieren
     * @param sql SQL Befehl der ausgef&uuml;hrt werden soll
     * @throws SQLException 
     */
    public void executeSql(String sql) throws SQLException;
    
    /**
     * Liefert eine Verbindung zu Datenbank (wird f&uuml;r Testdaten ben&ouml;tigt)
     * @return 
     * @throws SQLException 
     */
    public Connection getConnection() throws SQLException;
    
    
    /**
     * Liefert alle Exemplar-IDs, die es zu diesem Artikel gibt
     * @param artikel Artikel
     * @param withBestand Sollen Bestands-Exemplare mit ausgegeben werden?
     * @return 
     * @throws SQLException 
     */
    public List<Integer> getExemplarIds(Artikel artikel, boolean withBestand) throws SQLException;
    
    /**
     * Liefert eine Liste mit allen Reservierungen des Benutzers
     * @param user User
     * @return 
     * @throws SQLException
     */
    public List<Reservierung> getReserveFromUser(IUser user) throws SQLException;
    
    
    /**
     * Liefert die Filiale mit dem &uuml;bergeben Namen (case sensitiv)
     * @param name Name der Filiale
     * @return
     * @throws SQLException 
     */
    public Filiale getFilialeByName(String name) throws SQLException;
}
